@@ -2,27 +2,23 @@
 Dependency-Injected HTTP metadata.
 """
 
-from typing import Any, Dict, Mapping, Sequence, TYPE_CHECKING, Text, Union
+from typing import Any, Dict, Mapping, Sequence, Type, Union, cast
 
 import attr
-
 from hyperlink import DecodedURL
+from zope.interface import Interface, implementer, provider
 
-from six import text_type
+from twisted.python.components import Componentized
+from twisted.web.iweb import IRequest
 
-from zope.interface import implementer, provider
-from zope.interface.interfaces import IInterface
-
-from .interfaces import IDependencyInjector, IRequiredParameter
-
-if TYPE_CHECKING:  # pragma: no cover
-    from klein.interfaces import IRequestLifecycle
-    from twisted.web.iweb import IRequest
-    from twisted.python.components import Componentized
+from .interfaces import (
+    IDependencyInjector,
+    IRequestLifecycle,
+    IRequiredParameter,
+)
 
 
-def urlFromRequest(request):
-    # type: (IRequest) -> DecodedURL
+def urlFromRequest(request: IRequest) -> DecodedURL:
     sentHeader = request.getHeader(b"host")
     if sentHeader is not None:
         sentHeader = sentHeader.decode("charmap")
@@ -33,14 +29,13 @@ def urlFromRequest(request):
             host = sentHeader
             port = None
     else:
-        host = request.client.host
-        port = request.client.port
-        if not isinstance(host, text_type):
-            host = host.decode("ascii")
+        client = request.client  # type: ignore[attr-defined]
+        host = client.host
+        port = client.port
 
     url = DecodedURL.fromText(request.uri.decode("charmap"))
     url = url.replace(
-        scheme=u"https" if request.isSecure() else u"http",
+        scheme="https" if request.isSecure() else "http",
         host=host,
         port=port,
     )
@@ -48,7 +43,7 @@ def urlFromRequest(request):
 
 
 @provider(IRequiredParameter, IDependencyInjector)
-class RequestURL(object):
+class RequestURL:
     """
     Require a hyperlink L{DecodedURL} object from a L{Requirer}.
 
@@ -57,50 +52,61 @@ class RequestURL(object):
 
     @classmethod
     def registerInjector(
-        cls, injectionComponents, parameterName, requestLifecycle
-    ):
-        # type: (Componentized, str, IRequestLifecycle) -> IDependencyInjector
-        return cls()
+        cls,
+        injectionComponents: Componentized,
+        parameterName: str,
+        requestLifecycle: IRequestLifecycle,
+    ) -> IDependencyInjector:
+        # typing note: https://github.com/Shoobx/mypy-zope/issues/39
+        return cast(IDependencyInjector, cls())
 
     @classmethod
-    def injectValue(cls, instance, request, routeParams):
-        # type: (Any, IRequest, Dict[str, Any]) -> DecodedURL
+    def injectValue(
+        cls,
+        instance: Any,
+        request: IRequest,
+        routeParams: Dict[str, Any],
+    ) -> DecodedURL:
         return urlFromRequest(request)
 
     @classmethod
-    def finalize(cls):
-        # type: () -> None
+    def finalize(cls) -> None:
         "Nothing to do upon finalization."
 
 
 @implementer(IRequiredParameter, IDependencyInjector)
-@attr.s(frozen=True)
-class RequestComponent(object):
+@attr.frozen
+class RequestComponent:
     """
     Require a hyperlink L{DecodedURL} object from a L{Requirer}.
 
     @since: Klein NEXT
     """
 
-    interface = attr.ib(type=IInterface)
+    interface: Type[Interface]
 
     def registerInjector(
-        self, injectionComponents, parameterName, requestLifecycle
-    ):
-        # type: (Componentized, str, IRequestLifecycle) -> IDependencyInjector
+        self,
+        injectionComponents: Componentized,
+        parameterName: str,
+        requestLifecycle: IRequestLifecycle,
+    ) -> IDependencyInjector:
         return self
 
-    def injectValue(self, instance, request, routeParams):
-        # type: (Any, IRequest, Dict[str, Any]) -> DecodedURL
-        return request.getComponent(self.interface)
+    def injectValue(
+        self, instance: Any, request: IRequest, routeParams: Dict[str, Any]
+    ) -> DecodedURL:
+        return cast(
+            DecodedURL,
+            cast(Componentized, request).getComponent(self.interface),
+        )
 
-    def finalize(cls):
-        # type: () -> None
+    def finalize(cls) -> None:
         "Nothing to do upon finalization."
 
 
-@attr.s(frozen=True)
-class Response(object):
+@attr.s(auto_attribs=True, frozen=True)
+class Response:
     """
     Metadata about an HTTP response, with an object that Klein knows how to
     understand.
@@ -112,22 +118,18 @@ class Response(object):
         - some HTTP headers
 
         - a body object, which can be anything else Klein understands; for
-          example, an IResource, an IRenderable, text, bytes, etc.
+          example, an IResource, an IRenderable, str, bytes, etc.
 
     @since: Klein NEXT
     """
 
-    code = attr.ib(type=int, default=200)
-    headers = attr.ib(
-        type=Mapping[
-            Union[Text, bytes], Union[Text, bytes, Sequence[Union[Text, bytes]]]
-        ],
-        default=attr.Factory(dict),
-    )
-    body = attr.ib(type=Any, default=u"")
+    code: int = 200
+    headers: Mapping[
+        Union[str, bytes], Union[str, bytes, Sequence[Union[str, bytes]]]
+    ] = attr.ib(factory=dict)
+    body: Any = ""
 
-    def _applyToRequest(self, request):
-        # type: (IRequest) -> Any
+    def _applyToRequest(self, request: IRequest) -> Any:
         """
         Apply this L{Response} to the given L{IRequest}, setting its response
         code and headers.
@@ -141,7 +143,7 @@ class Response(object):
         """
         request.setResponseCode(self.code)
         for headerName, headerValueOrValues in self.headers.items():
-            if not isinstance(headerValueOrValues, (text_type, bytes)):
+            if not isinstance(headerValueOrValues, (str, bytes)):
                 headerValues = headerValueOrValues
             else:
                 headerValues = [headerValueOrValues]
