@@ -1,23 +1,23 @@
 # -*- test-case-name: klein.test.test_form -*-
 
-from __future__ import print_function, unicode_literals
-
 import json
 from typing import (
     Any,
     AnyStr,
     Callable,
     Dict,
+    Generator,
     Iterable,
     List,
+    NoReturn,
     Optional,
     Sequence,
-    Text,
     Type,
     cast,
 )
 
 import attr
+from zope.interface import Attribute, Interface, implementer
 
 from twisted.internet.defer import Deferred, inlineCallbacks
 from twisted.python.components import Componentized, registerAdapter
@@ -27,11 +27,8 @@ from twisted.web.iweb import IRenderable, IRequest
 from twisted.web.resource import Resource
 from twisted.web.template import Element, Tag, TagLoader, tags
 
-from zope.interface import Interface, implementer
-
-from ._app import _call
+from ._app import KleinRenderable, _call
 from ._decorators import bindable
-from ._typing import DefaultNamedArg, NoReturn
 from .interfaces import (
     EarlyExit,
     IDependencyInjector,
@@ -44,18 +41,16 @@ from .interfaces import (
 )
 
 
-class CrossSiteRequestForgery(Resource, object):
+class CrossSiteRequestForgery(Resource):
     """
     Cross site request forgery detected.  Request aborted.
     """
 
-    def __init__(self, message):
-        # type: (str) -> None
-        super(CrossSiteRequestForgery, self).__init__()
+    def __init__(self, message: str) -> None:
+        super().__init__()
         self.message = message
 
-    def render(self, request):
-        # type: (IRequest) -> bytes
+    def render(self, request: IRequest) -> bytes:
         """
         For all HTTP methods, return a 403.
         """
@@ -66,8 +61,7 @@ class CrossSiteRequestForgery(Resource, object):
 CSRF_PROTECTION = "__csrf_protection__"
 
 
-def textConverter(value):
-    # type: (AnyStr) -> Text
+def textConverter(value: AnyStr) -> str:
     """
     Converter for form values (which may be any type of string) into text.
     """
@@ -87,30 +81,32 @@ class IParsedJSONBody(Interface):
 
 
 @implementer(IRequiredParameter)
-@attr.s(frozen=True)
-class Field(object):
+@attr.s(auto_attribs=True, frozen=True)
+class Field:
     """
     A L{Field} is a static part of a L{Form}.
 
     @ivar converter: The converter.
     """
 
-    converter = attr.ib(type=Callable[[AnyStr], Any])
-    formInputType = attr.ib(type=str)
-    pythonArgumentName = attr.ib(type=Optional[str], default=None)
-    formFieldName = attr.ib(type=Optional[str], default=None)
-    formLabel = attr.ib(type=Optional[str], default=None)
-    default = attr.ib(type=Optional[Any], default=None, cmp=False)
-    required = attr.ib(type=bool, default=True)
-    noLabel = attr.ib(type=bool, default=False)
-    value = attr.ib(type=Text, default="")
-    error = attr.ib(type=ValidationError, default=None)
+    converter: Callable[[AnyStr], Any]
+    formInputType: str
+    pythonArgumentName: Optional[str] = None
+    formFieldName: Optional[str] = None
+    formLabel: Optional[str] = None
+    default: Optional[Any] = attr.ib(default=None, cmp=False)
+    required: bool = True
+    noLabel: bool = False
+    value: str = ""
+    error: Optional[ValidationError] = None
 
     # IRequiredParameter
     def registerInjector(
-        self, injectionComponents, parameterName, requestLifecycle
-    ):
-        # type: (Componentized, str, IRequestLifecycle) -> IDependencyInjector
+        self,
+        injectionComponents: Componentized,
+        parameterName: str,
+        requestLifecycle: IRequestLifecycle,
+    ) -> IDependencyInjector:
         """
         Register this form field as a dependency injector.
         """
@@ -120,18 +116,17 @@ class Field(object):
             protoForm.addField(self.maybeNamed(parameterName)),
         )
 
-    def maybeNamed(self, name):
-        # type: (str) -> Field
+    def maybeNamed(self, name: str) -> "Field":
         """
         Create a new L{Field} like this one, but with all the name default
         values filled in.
 
         @param name: the name.
-        @type name: a native L{str}
         """
 
-        def maybe(it, that=name):
-            # type: (Optional[str], Optional[str]) -> Optional[str]
+        def maybe(
+            it: Optional[str], that: Optional[str] = name
+        ) -> Optional[str]:
             return that if it is None else it
 
         return attr.assoc(
@@ -143,41 +138,40 @@ class Field(object):
             ),
         )
 
-    def asTags(self):
-        # type: () -> Iterable[Tag]
+    def asTags(self) -> Iterable[Tag]:
         """
         Convert this L{Field} into some stuff that can be rendered in a
         L{twisted.web.template}.
 
         @return: A new set of tags to include in a template.
-        @rtype: iterable of L{twisted.web.template.Tag}
         """
         value = self.value
         if value is None:
             value = ""  # type: ignore[unreachable]
         input_tag = tags.input(
-            type=self.formInputType, name=self.formFieldName, value=value
+            type=self.formInputType,
+            name=self.formFieldName,  # type: ignore[arg-type]
+            value=value,
         )
         error_tags = []
         if self.error:
             error_tags.append(
                 tags.div(class_="klein-form-validation-error")(
-                    self.error.message
+                    self.error.message  # type: ignore[arg-type]
                 )
             )
         if self.formLabel:
             yield tags.label(self.formLabel, ": ", input_tag, *error_tags)
         else:
             yield input_tag
-            yield error_tags
+            yield from error_tags
 
-    def extractValue(self, request):
-        # type: (IRequest) -> Any
+    def extractValue(self, request: IRequest) -> Any:
         """
         Extract a value from the request.
 
         In the case of key/value form posts, this attempts to reliably make the
-        value into Text.  In the case of a JSON post, however, it will simply
+        value into str.  In the case of a JSON post, however, it will simply
         extract the value from the top-level dictionary, which means it could
         be any arrangement of JSON-serializiable objects.
         """
@@ -189,13 +183,15 @@ class Field(object):
             b"application/json"
         ):
             # TODO: parse only once, please.
-            parsed = request.getComponent(IParsedJSONBody)
+            parsed = cast(Componentized, request).getComponent(IParsedJSONBody)
             if parsed is None:
                 request.content.seek(0)
                 octets = request.content.read()
                 characters = octets.decode("utf-8")
                 parsed = json.loads(characters)
-                request.setComponent(IParsedJSONBody, parsed)
+                cast(Componentized, request).setComponent(
+                    IParsedJSONBody, parsed
+                )
             if fieldName not in parsed:
                 return None
             return parsed[fieldName]
@@ -205,8 +201,7 @@ class Field(object):
         else:
             return None
 
-    def validateValue(self, value):
-        # type: (Any) -> Any
+    def validateValue(self, value: Any) -> Any:
         """
         Validate the given text and return a converted Python object to use, or
         fail with L{ValidationError}.
@@ -226,7 +221,7 @@ class Field(object):
             raise ValidationError(str(ve))
 
     @classmethod
-    def text(cls, **kw):  # type: (**Any) -> Field
+    def text(cls, **kw: Any) -> "Field":
         """
         Shorthand for a form field that contains a short string, and will be
         rendered as a plain <input>.
@@ -234,7 +229,7 @@ class Field(object):
         return cls(converter=textConverter, formInputType="text", **kw)
 
     @classmethod
-    def password(cls, **kw):  # type: (**Any) -> Field
+    def password(cls, **kw: Any) -> "Field":
         """
         Shorthand for a form field that, like L{text}, contains a short string,
         but should be obscured when typed (and, to the extent possible,
@@ -243,8 +238,7 @@ class Field(object):
         return cls(converter=textConverter, formInputType="password", **kw)
 
     @classmethod
-    def hidden(cls, name, value, **kw):
-        # type: (str, Text, **Any) -> Field
+    def hidden(cls, name: str, value: str, **kw: Any) -> "Field":
         """
         Shorthand for a hidden field.
         """
@@ -253,18 +247,22 @@ class Field(object):
             formInputType="hidden",
             noLabel=True,
             value=value,
-            **kw
+            **kw,
         ).maybeNamed(name)
 
     @classmethod
-    def number(cls, minimum=None, maximum=None, kind=float, **kw):
-        # type: (Optional[int], Optional[int], Type, **Any) -> Field
+    def number(
+        cls,
+        minimum: Optional[int] = None,
+        maximum: Optional[int] = None,
+        kind: Type = float,
+        **kw: Any,
+    ) -> "Field":
         """
         An integer within the range [minimum, maximum].
         """
 
-        def bounded_number(text):
-            # type: (AnyStr) -> Any
+        def bounded_number(text: AnyStr) -> Any:
             try:
                 value = kind(text)
             except (ValueError, ArithmeticError):
@@ -279,8 +277,7 @@ class Field(object):
         return cls(converter=bounded_number, formInputType="number", **kw)
 
     @classmethod
-    def submit(cls, value):
-        # type: (Text) -> Field
+    def submit(cls, value: str) -> "Field":
         """
         A field representing a submit button, with a value (displayed on the
         button).
@@ -294,48 +291,40 @@ class Field(object):
 
 
 @implementer(IRenderable)
-@attr.s
-class RenderableForm(object):
+@attr.s(auto_attribs=True)
+class RenderableForm:
     """
     An L{IRenderable} representing a renderable form.
 
     @ivar prevalidationValues: a L{dict} mapping {L{Field}: L{list} of
-        L{Text}}, representing the value that each field received as part of
+        L{str}}, representing the value that each field received as part of
         the request.
 
     @ivar validationErrors: a L{dict} mapping {L{Field}: L{ValidationError}}
     """
 
-    _form = attr.ib(type="Form")
-    _session = attr.ib(type=ISession)
-    _action = attr.ib(type=str)
-    _method = attr.ib(type=str)
-    _enctype = attr.ib(type=str)
-    _encoding = attr.ib(type=str)
-    prevalidationValues = attr.ib(
-        type=Dict[Field, Optional[Text]],
-        default=cast(Dict[Field, Optional[Text]], attr.Factory(dict)),
-    )
-    validationErrors = attr.ib(
-        type=Dict[Field, ValidationError],
-        default=cast(Dict[Field, ValidationError], attr.Factory(dict)),
-    )
+    _form: "IForm"
+    _session: ISession
+    _action: str
+    _method: str
+    _enctype: str
+    _encoding: str
+    prevalidationValues: Dict[Field, Optional[str]] = attr.ib(factory=dict)
+    validationErrors: Dict[Field, ValidationError] = attr.ib(factory=dict)
 
     ENCTYPE_FORM_DATA = "multipart/form-data"
     ENCTYPE_URL_ENCODED = "application/x-www-form-urlencoded"
 
-    def _fieldForCSRF(self):
-        # type: () -> Field
+    def _fieldForCSRF(self) -> Field:
         """
         @return: A hidden L{Field} containing the cross-site request forgery
             protection token.
         """
         return Field.hidden(CSRF_PROTECTION, self._session.identifier)
 
-    def _fieldsToRender(self):
-        # type: () -> Iterable[Field]
+    def _fieldsToRender(self) -> Iterable[Field]:
         """
-        @return: an interable of L{Field} objects to include in the HTML
+        @return: an iterable of L{Field} objects to include in the HTML
             representation of this form.  This includes:
 
                 - all the user-specified fields in the form
@@ -367,16 +356,14 @@ class RenderableForm(object):
 
     # Public interface below.
 
-    def lookupRenderMethod(self, name):
-        # type: (str) -> NoReturn
+    def lookupRenderMethod(self, name: str) -> NoReturn:
         """
         Form renderers don't supply any render methods, so this just always
         raises L{MissingRenderMethod}.
         """
         raise MissingRenderMethod(self, name)
 
-    def render(self, request):
-        # type: (IRequest) -> Tag
+    def render(self, request: IRequest) -> Tag:  # type: ignore[override]
         """
         Render this form to the given request.
         """
@@ -391,8 +378,7 @@ class RenderableForm(object):
             action=self._action, method=self._method, **formAttributes
         )(field.asTags() for field in self._fieldsToRender())
 
-    def glue(self):
-        # type: () -> Iterable[Tag]
+    def glue(self) -> List[Tag]:
         """
         Provide any glue necessary to render this form; this must be dropped
         into the template within the C{<form>} tag.
@@ -403,18 +389,16 @@ class RenderableForm(object):
 
         @return: some HTML elements in the form of renderable objects for
             L{twisted.web.template}
-        @rtype: L{twisted.web.template.Tag}, or L{list} thereof.
         """
-        return self._fieldForCSRF().asTags()
+        return list(self._fieldForCSRF().asTags())
 
 
 @bindable
 def defaultValidationFailureHandler(
-    instance,  # type: Optional[object]
-    request,  # type: IRequest
-    fieldValues,  # type: FieldValues
-):
-    # type: (...) -> Element
+    instance: Optional[object],
+    request: IRequest,
+    fieldValues: "FieldValues",
+) -> Element:
     """
     This is the default validation failure handler, which will be used by form
     handlers (i.e. any routes which use L{klein.Requirer} to require a field)
@@ -426,14 +410,10 @@ def defaultValidationFailureHandler(
 
     @param instance: The instance associated with the router that the form
         handler was handled on.
-    @type instance: L{object}
-
     @param request: The request including the form submission.
-    @type request: L{twisted.web.iweb.IRequest}
-
     @return: Any object acceptable from a Klein route.
     """
-    session = request.getComponent(ISession)
+    session = cast(Componentized, request).getComponent(ISession)
     request.setResponseCode(400)
     enctype = (
         (
@@ -462,14 +442,6 @@ def defaultValidationFailureHandler(
 
 _requirerFunctionWithForm = Any
 _routeCallable = Any
-_routeDecorator = Callable[
-    [_routeCallable, DefaultNamedArg(Any, "__session__")], _routeCallable
-]
-_validationFailureHandler = Callable[
-    [Optional[object], IRequest, "Form", Dict[str, str]], Element
-]
-
-validationFailureHandlerAttribute = "__kleinFormValidationFailureHandlers__"
 
 
 class IProtoForm(Interface):
@@ -477,40 +449,54 @@ class IProtoForm(Interface):
     Marker interface for L{ProtoForm}.
     """
 
+    fields: Sequence[Field] = Attribute("Form fields")
+
+    def addField(field: Field) -> "FieldInjector":
+        """
+        Add the given field to the form ultimately created here.
+        """
+
 
 class IForm(Interface):
     """
     Marker interface for form attached to dependency injection components.
     """
 
+    fields: Sequence[Field] = Attribute("Form fields")
+
+    def populateRequestValues(
+        injectionComponents: Componentized,
+        instance: Any,
+        request: IRequest,
+    ) -> Deferred:
+        """
+        Extract the values present in this request and populate a
+        L{FieldValues} object.
+        """
+
 
 @implementer(IProtoForm)
-@attr.s
-class ProtoForm(object):
+@attr.s(auto_attribs=True)
+class ProtoForm:
     """
     Form-builder.
     """
 
-    _componentized = attr.ib(type=Componentized)
-    _lifecycle = attr.ib(type=IRequestLifecycle)
-    _fields = attr.ib(type=List[Field], default=attr.Factory(list))
+    _componentized: Componentized
+    _lifecycle: IRequestLifecycle
+    fields: List[Field] = attr.ib(factory=list)
 
     @classmethod
-    def fromComponentized(cls, componentized):
-        # type: (Componentized) -> ProtoForm
+    def fromComponentized(cls, componentized: Componentized) -> "ProtoForm":
         """
         Create a ProtoForm from a componentized object.
         """
-        rl = IRequestLifecycle(componentized)  # type: ignore[operator]
+        rl = IRequestLifecycle(componentized)
         assert rl is not None
         return cls(componentized, rl)
 
-    def addField(self, field):
-        # type: (Field) -> FieldInjector
-        """
-        Add the given field to the form ultimately created here.
-        """
-        self._fields.append(field)
+    def addField(self, field: Field) -> "FieldInjector":
+        self.fields.append(field)
         return FieldInjector(self._componentized, field, self._lifecycle)
 
 
@@ -519,67 +505,86 @@ class IFieldValues(Interface):
     Marker interface for parsed fields.
     """
 
+    form: IForm = Attribute("Form")
+    arguments: Dict[str, Any] = Attribute("Arguments")
+    prevalidationValues: Dict[Field, Optional[str]] = Attribute(
+        "Pre-validation values"
+    )
+    validationErrors: Dict[Field, ValidationError] = Attribute(
+        "Validation errors"
+    )
+
+    def validate(instance: Any, request: IRequest) -> Deferred:
+        """
+        If any validation errors have occurred, raise a relevant exception.
+        """
+
 
 @implementer(IFieldValues)
-@attr.s
-class FieldValues(object):
+@attr.s(auto_attribs=True)
+class FieldValues:
     """
     Reified post-parsing values for HTTP form submission.
     """
 
-    form = attr.ib(type="Form")
-    arguments = attr.ib(type=Dict[str, Any])
-    prevalidationValues = attr.ib(type=Dict[Field, Optional[Text]])
-    validationErrors = attr.ib(type=Dict[Field, ValidationError])
-    _injectionComponents = attr.ib(type=Componentized)
+    form: "Form"
+    arguments: Dict[str, Any]
+    prevalidationValues: Dict[Field, Optional[str]]
+    validationErrors: Dict[Field, ValidationError]
+    _injectionComponents: Componentized
 
     @inlineCallbacks
-    def validate(self, instance, request):
-        # type: (Any, IRequest) -> Deferred
-        """
-        If any validation errors have occurred, raise a relevant exception.
-        """
+    def validate(
+        self, instance: Any, request: IRequest
+    ) -> Generator[Any, object, None]:
         if self.validationErrors:
-            result = yield _call(
-                instance,
-                IValidationFailureHandler(
-                    self._injectionComponents, defaultValidationFailureHandler
+            result = cast(
+                KleinRenderable,
+                (
+                    yield _call(
+                        instance,
+                        IValidationFailureHandler(
+                            self._injectionComponents,
+                            defaultValidationFailureHandler,
+                        ),
+                        request,
+                        self,
+                    )
                 ),
-                request,
-                self,
             )
             raise EarlyExit(result)
 
 
 @implementer(IDependencyInjector)
-@attr.s
-class FieldInjector(object):
+@attr.s(auto_attribs=True)
+class FieldInjector:
     """
     Field injector.
     """
 
-    _componentized = attr.ib(type=Componentized)
-    _field = attr.ib(type=Field)
-    _lifecycle = attr.ib(type=IRequestLifecycle)
+    _componentized: Componentized
+    _field: Field
+    _lifecycle: IRequestLifecycle
 
-    def injectValue(self, instance, request, routeParams):
-        # type: (Any, IRequest, Dict[str, Any]) -> Any
+    def injectValue(
+        self, instance: Any, request: IRequest, routeParams: Dict[str, Any]
+    ) -> Any:
         """
         Inject the given value into the form.
         """
+        assert self._field.pythonArgumentName is not None
         return IFieldValues(request).arguments.get(
             self._field.pythonArgumentName
         )
 
-    def finalize(self):
-        # type: () -> None
+    def finalize(self) -> None:
         """
         Finalize this ProtoForm into a real form.
         """
         if IForm(self._componentized, None) is not None:
             return
 
-        finalForm = cast(IForm, Form(IProtoForm(self._componentized)._fields))
+        finalForm = cast(IForm, Form(IProtoForm(self._componentized).fields))
         self._componentized.setComponent(IForm, finalForm)
 
         # XXX set requiresComponents argument here to ISession if CSRF is
@@ -587,14 +592,15 @@ class FieldInjector(object):
         # side-effect-free (like a search field) that can be handled even
         # without a CSRF token.
         @bindable
-        def populateValuesHook(instance, request):
-            # type: (Any, IRequest) -> Deferred
+        def populateValuesHook(instance: Any, request: IRequest) -> Deferred:
             return finalForm.populateRequestValues(
                 self._componentized, instance, request
             )
 
         self._lifecycle.addPrepareHook(
-            populateValuesHook, provides=[IFieldValues], requires=[ISession],
+            populateValuesHook,
+            provides=[IFieldValues],
+            requires=[ISession],
         )
 
 
@@ -606,15 +612,19 @@ class IValidationFailureHandler(Interface):
     Validation failure handler callable interface.
     """
 
+    def __call__(request: IRequest) -> KleinRenderable:
+        """
+        Called to handle a validation failure.
+        """
 
-def checkCSRF(request):
-    # type: (IRequest) -> None
+
+def checkCSRF(request: IRequest) -> None:
     """
     Check the request for cross-site request forgery, raising an EarlyExit if
     it is found.
     """
     # TODO: optionalize CSRF protection for GET forms
-    session = ISession(request, None)  # type: ignore[operator]
+    session = ISession(request, None)
     token = None
     if request.method in (b"GET", b"HEAD"):
         # Idempotent requests don't require CRSF validation.  (Don't have
@@ -636,22 +646,22 @@ def checkCSRF(request):
             return
     # leak only the value passed, not the actual token, just in
     # case there's some additional threat vector there
-    raise EarlyExit(
-        CrossSiteRequestForgery("Invalid CSRF token: {!r}".format(token))
-    )
+    raise EarlyExit(CrossSiteRequestForgery(f"Invalid CSRF token: {token!r}"))
 
 
-@attr.s(hash=False)
-class Form(object):
+@implementer(IForm)
+@attr.s(auto_attribs=True, hash=False)
+class Form:
     """
     A L{Form} is a collection of fields attached to a function.
     """
 
-    fields = attr.ib(type=Sequence[Field])
+    fields: Sequence[Field]
 
     @staticmethod
-    def onValidationFailureFor(handler):
-        # type: (_requirerFunctionWithForm) -> Callable[[Callable], Callable]
+    def onValidationFailureFor(
+        handler: _requirerFunctionWithForm,
+    ) -> Callable[[Callable], Callable]:
         """
         Register a function to be run in the event of a validation failure for
         the input to a particular form handler.
@@ -684,8 +694,7 @@ class Form(object):
             C{(request, form) -> thing klein can render}.
         """
 
-        def decorate(decoratee):
-            # type: (Callable) -> Callable
+        def decorate(decoratee: Callable) -> Callable:
             handler.injectionComponents.setComponent(
                 IValidationFailureHandler, decoratee
             )
@@ -694,12 +703,12 @@ class Form(object):
         return decorate
 
     @inlineCallbacks
-    def populateRequestValues(self, injectionComponents, instance, request):
-        # type: (Componentized, Any, IRequest) -> Deferred
-        """
-        Extract the values present in this request and populate a
-        L{FieldValues} object.
-        """
+    def populateRequestValues(
+        self,
+        injectionComponents: Componentized,
+        instance: Any,
+        request: IRequest,
+    ) -> Generator[Any, object, None]:
         assert IFieldValues(request, None) is None
 
         validationErrors = {}
@@ -728,25 +737,24 @@ class Form(object):
             injectionComponents,
         )
         yield values.validate(instance, request)
-        request.setComponent(IFieldValues, values)
+        cast(Componentized, request).setComponent(IFieldValues, values)
 
     @classmethod
     def rendererFor(
         cls,
-        decoratedFunction,  # type: _requirerFunctionWithForm
-        action,  # type: Text
-        method="POST",  # type: Text
-        enctype=RenderableForm.ENCTYPE_FORM_DATA,  # type: Text
-        encoding="utf-8",  # type: str
-    ):
-        # type: (...) -> RenderableFormParam
+        decoratedFunction: _requirerFunctionWithForm,
+        action: str,
+        method: str = "POST",
+        enctype: str = RenderableForm.ENCTYPE_FORM_DATA,
+        encoding: str = "utf-8",
+    ) -> "RenderableFormParam":
         """
         A form parameter that can render a form declared as a number of fields
         on another route.
 
         Use like so::
 
-            class MyFormApp(object):
+            class MyFormApp:
                 router = Klein()
                 requirer = Requirer()
 
@@ -767,42 +775,45 @@ class Form(object):
         As a L{RenderableForm} provides L{IRenderable}, you may return the
         parameter directly
         """
-        form = IForm(
+        form: Optional[IForm] = IForm(
             decoratedFunction.injectionComponents, None
-        )  # type: Optional[Form]
+        )
         if form is None:
             form = Form([])
         return RenderableFormParam(form, action, method, enctype, encoding)
 
 
 @implementer(IRequiredParameter, IDependencyInjector)
-@attr.s
-class RenderableFormParam(object):
+@attr.s(auto_attribs=True)
+class RenderableFormParam:
     """
     A L{RenderableFormParam} implements L{IRequiredParameter} and
     L{IDependencyInjector} to provide a L{RenderableForm} to your route.
     """
 
-    _form = attr.ib(type=Form)
-    _action = attr.ib(type=Text)
-    _method = attr.ib(type=Text)
-    _enctype = attr.ib(type=Text)
-    _encoding = attr.ib(type=Text)
+    _form: IForm
+    _action: str
+    _method: str
+    _enctype: str
+    _encoding: str
 
     def registerInjector(
-        self, injectionComponents, parameterName, requestLifecycle
-    ):
-        # type: (Componentized, str, IRequestLifecycle) -> RenderableFormParam
+        self,
+        injectionComponents: Componentized,
+        parameterName: str,
+        requestLifecycle: IRequestLifecycle,
+    ) -> "RenderableFormParam":
         return self
 
-    def injectValue(self, instance, request, routeParams):
-        # type: (Any, IRequest, Dict[str, Any]) -> RenderableForm
+    def injectValue(
+        self, instance: Any, request: IRequest, routeParams: Dict[str, Any]
+    ) -> RenderableForm:
         """
         Create the renderable form from the request.
         """
         return RenderableForm(
             self._form,
-            ISession(request),  # type: ignore[operator]
+            ISession(request),
             self._action,
             self._method,
             self._enctype,
@@ -811,8 +822,7 @@ class RenderableFormParam(object):
             validationErrors={},
         )
 
-    def finalize(self):
-        # type: () -> None
+    def finalize(self) -> None:
         """
         Nothing to do upon finalization.
         """
