@@ -1,5 +1,5 @@
 # -*- test-case-name: klein.test -*-
-# Copyright (c) 2011-2019. See LICENSE for details.
+# Copyright (c) 2011-2021. See LICENSE for details.
 
 """
 Tests for L{klein}.
@@ -7,7 +7,12 @@ Tests for L{klein}.
 
 from hypothesis import HealthCheck, settings
 
+
 settings.register_profile(
-    "patience", settings(suppress_health_check=[HealthCheck.too_slow])
+    "patience",
+    settings(
+        deadline=None,
+        suppress_health_check=[HealthCheck.too_slow],
+    ),
 )
 settings.load_profile("patience")
