@@ -1,17 +1,17 @@
-from typing import List, TYPE_CHECKING, Text, cast
+from typing import Any, List, Tuple, cast
 from xml.etree import ElementTree
 
 import attr
-
 from treq import content
 from treq.testing import StubTreq
 
 from twisted.internet.defer import inlineCallbacks
 from twisted.python.compat import nativeString
 from twisted.trial.unittest import SynchronousTestCase
+from twisted.web.iweb import IRequest
 from twisted.web.template import Element, TagLoader, renderer, tags
 
-from klein import Field, Form, Klein, Requirer, SessionProcurer
+from klein import Field, Form, Klein, RenderableForm, Requirer, SessionProcurer
 from klein.interfaces import (
     ISession,
     ISessionStore,
@@ -23,11 +23,6 @@ from klein.storage.memory import MemorySessionStore
 
 from .._form import textConverter
 
-if TYPE_CHECKING:  # pragma: no cover
-    from typing import Any, Tuple
-    from twisted.web.iweb import IRequest
-    from klein import RenderableForm
-
 
 class DanglingField(Field):
     """
@@ -35,23 +30,21 @@ class DanglingField(Field):
     told.
     """
 
-    def maybeNamed(self, name):
-        # type: (Text) -> Field
+    def maybeNamed(self, name: str) -> Field:
         return self
 
 
-@attr.s(hash=False)
-class TestObject(object):
-    sessionStore = attr.ib(type=ISessionStore)
-    calls = attr.ib(attr.Factory(list), type=List)
+@attr.s(auto_attribs=True, hash=False)
+class TestObject:
+    sessionStore: ISessionStore
+    calls: List = attr.ib(factory=list)
 
     router = Klein()
     requirer = Requirer()
 
     @requirer.prerequisite([ISession])
     @inlineCallbacks
-    def procureASession(self, request):
-        # type: (IRequest) -> Any
+    def procureASession(self, request: IRequest) -> Any:
         try:
             yield (
                 SessionProcurer(
@@ -68,8 +61,7 @@ class TestObject(object):
         router.route("/dangling-param", methods=["POST"]),
         dangling=DanglingField(lambda x: x, "text"),
     )
-    def danglingParameter(self, dangling):
-        # type: (str) -> None
+    def danglingParameter(self, dangling: str) -> None:
         "..."
 
     @requirer.require(
@@ -77,18 +69,16 @@ class TestObject(object):
         name=Field.text(),
         value=Field.number(),
     )
-    def handler(self, name, value):
-        # type: (Text, float) -> bytes
+    def handler(self, name: str, value: float) -> bytes:
         self.calls.append((name, value))
         return b"yay"
 
     @requirer.require(
         router.route("/handle-submit", methods=["POST"]),
         name=Field.text(),
-        button=Field.submit(u"OK"),
+        button=Field.submit("OK"),
     )
-    def handlerWithSubmit(self, name, button):
-        # type: (str, str) -> None
+    def handlerWithSubmit(self, name: str, button: str) -> None:
         """
         Form with a submit button.
         """
@@ -96,8 +86,7 @@ class TestObject(object):
     @requirer.require(
         router.route("/password-field", methods=["POST"]), pw=Field.password()
     )
-    def gotPassword(self, pw):
-        # type: (Text) -> bytes
+    def gotPassword(self, pw: str) -> bytes:
         self.calls.append(("password", pw))
         return b"password received"
 
@@ -106,8 +95,7 @@ class TestObject(object):
         name=Field.text(),
         value=Field.number(required=False, default=7.0),
     )
-    def notRequired(self, name, value):
-        # type: (IRequest, Text, float) -> bytes
+    def notRequired(self, name: str, value: float) -> bytes:
         self.calls.append((name, value))
         return b"okay"
 
@@ -115,33 +103,29 @@ class TestObject(object):
         router.route("/constrained", methods=["POST"]),
         goldilocks=Field.number(minimum=3, maximum=9),
     )
-    def constrained(self, goldilocks):
-        # type: (int) -> bytes
+    def constrained(self, goldilocks: int) -> bytes:
         self.calls.append(("constrained", goldilocks))
         return b"got it"
 
     @requirer.require(
         router.route("/render", methods=["GET"]),
-        form=Form.rendererFor(handler, action=u"/handle"),
+        form=Form.rendererFor(handler, action="/handle"),
     )
-    def renderer(self, form):
-        # type: (IRequest, Form) -> Form
+    def renderer(self, form: Form) -> Form:
         return form
 
     @requirer.require(
         router.route("/render-submit", methods=["GET"]),
-        form=Form.rendererFor(handlerWithSubmit, action=u"/handle-submit"),
+        form=Form.rendererFor(handlerWithSubmit, action="/handle-submit"),
     )
-    def submitRenderer(self, form):
-        # type: (IRequest, RenderableForm) -> RenderableForm
+    def submitRenderer(self, form: RenderableForm) -> RenderableForm:
         return form
 
     @requirer.require(
         router.route("/render-custom", methods=["GET"]),
-        form=Form.rendererFor(handler, action=u"/handle"),
+        form=Form.rendererFor(handler, action="/handle"),
     )
-    def customFormRender(self, form):
-        # type: (RenderableForm) -> Any
+    def customFormRender(self, form: RenderableForm) -> Any:
         """
         Include just the glue necessary for CSRF protection and let the
         application render the rest of the form.
@@ -150,15 +134,12 @@ class TestObject(object):
 
     @requirer.require(
         router.route("/render-cascade", methods=["GET"]),
-        form=Form.rendererFor(handler, action=u"/handle"),
+        form=Form.rendererFor(handler, action="/handle"),
     )
-    def cascadeRenderer(self, form):
-        # type: (RenderableForm) -> RenderableForm
-
+    def cascadeRenderer(self, form: RenderableForm) -> Element:
         class CustomElement(Element):
             @renderer
-            def customize(self, request, tag):
-                # type: (IRequest, Any) -> Any
+            def customize(self, request: IRequest, tag: Any) -> Any:
                 return tag("customized")
 
         form.validationErrors[form._form.fields[0]] = ValidationError(
@@ -171,39 +152,36 @@ class TestObject(object):
         router.route("/handle-validation", methods=["POST"]),
         value=Field.number(maximum=10),
     )
-    def customValidation(self, value):
-        # type: (int) -> None
+    def customValidation(self, value: int) -> None:
         """
         never called.
         """
 
     @requirer.require(Form.onValidationFailureFor(customValidation))
-    def customFailureHandling(self, values):
-        # type: (RenderableForm) -> bytes
+    def customFailureHandling(self, values: RenderableForm) -> bytes:
         """
         Handle validation failure.
         """
         self.calls.append(("validation", values))
         return b"~special~"
 
-    @requirer.require(router.route("/handle-empty", methods=["POST"]),)
-    def emptyHandler(self):
-        # type: () -> bytes
+    @requirer.require(
+        router.route("/handle-empty", methods=["POST"]),
+    )
+    def emptyHandler(self) -> bytes:
         """
         Empty form handler; just for testing rendering.
         """
 
     @requirer.require(
         router.route("/render-empty", methods=["GET"]),
-        form=Form.rendererFor(emptyHandler, action=u"/handle-empty"),
+        form=Form.rendererFor(emptyHandler, action="/handle-empty"),
     )
-    def emptyRenderer(self, form):
-        # type: (RenderableForm) -> RenderableForm
+    def emptyRenderer(self, form: RenderableForm) -> RenderableForm:
         return form
 
 
-def simpleFormRouter():
-    # type: () -> Tuple[Klein, List[Tuple[str, int]]]
+def simpleFormRouter() -> Tuple[Klein, List[Tuple[str, int]]]:
     """
     Create a simple router hooked up to a field handler.
     """
@@ -217,8 +195,7 @@ def simpleFormRouter():
         value=Field.number(),
         custom=Field(formInputType="number", converter=int, required=False),
     )
-    def justGet(name, value, custom):
-        # type: (str, int, int) -> bytes
+    def justGet(name: str, value: int, custom: int) -> bytes:
         calls.append((name, value or custom))
         return b"got"
 
@@ -230,19 +207,17 @@ class TestForms(SynchronousTestCase):
     Tests for L{klein.Form} and associated tools.
     """
 
-    def test_textConverter(self):
-        # type: () -> None
+    def test_textConverter(self) -> None:
         """
         Convert a string of either type to text.
         """
-        text = u"f\xf6o"
+        text = "f\xf6o"
         for string in (text, text.encode("utf-8")):
             result = textConverter(string)  # type: ignore[type-var]
-            self.assertIsInstance(result, Text)
+            self.assertIsInstance(result, str)
             self.assertEqual(result, text)
 
-    def test_handling(self):
-        # type: () -> None
+    def test_handling(self) -> None:
         """
         A handler for a Form with Fields receives those fields as input, as
         passed by an HTTP client.
@@ -264,10 +239,9 @@ class TestForms(SynchronousTestCase):
         )
         self.assertEqual(response.code, 200)
         self.assertEqual(self.successResultOf(content(response)), b"yay")
-        self.assertEqual(to.calls, [(u"hello", 1234)])
+        self.assertEqual(to.calls, [("hello", 1234)])
 
-    def test_handlingPassword(self):
-        # type: () -> None
+    def test_handlingPassword(self) -> None:
         """
         From the perspective of form handling, passwords are handled like
         strings.
@@ -291,10 +265,9 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(
             self.successResultOf(content(response)), b"password received"
         )
-        self.assertEqual(to.calls, [(u"password", u"asdfjkl;")])
+        self.assertEqual(to.calls, [("password", "asdfjkl;")])
 
-    def test_numberConstraints(self):
-        # type: () -> None
+    def test_numberConstraints(self) -> None:
         """
         Number parameters have minimum and maximum validations and the object
         will not be called when the values exceed them.
@@ -333,10 +306,9 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(tooLow.code, 400)
         self.assertEqual(justRight.code, 200)
         self.assertEqual(self.successResultOf(content(justRight)), b"got it")
-        self.assertEqual(to.calls, [(u"constrained", 7)])
+        self.assertEqual(to.calls, [("constrained", 7)])
 
-    def test_missingRequiredParameter(self):
-        # type: () -> None
+    def test_missingRequiredParameter(self) -> None:
         """
         If required fields are missing, a default error form is presented and
         the form's handler is not called.
@@ -363,8 +335,7 @@ class TestForms(SynchronousTestCase):
         )
         self.assertEqual(to.calls, [])
 
-    def test_noName(self):
-        # type: () -> None
+    def test_noName(self) -> None:
         """
         A handler for a Form with a Field that doesn't have a name will return
         an error explaining the problem.
@@ -389,8 +360,7 @@ class TestForms(SynchronousTestCase):
             str(errors[0].value), "Cannot extract unnamed form field."
         )
 
-    def test_handlingGET(self):
-        # type: () -> None
+    def test_handlingGET(self) -> None:
         """
         A GET handler for a Form with Fields receives query parameters matching
         those field names as input.
@@ -406,10 +376,9 @@ class TestForms(SynchronousTestCase):
         )
         self.assertEqual(response.code, 200)
         self.assertEqual(self.successResultOf(content(response)), b"got")
-        self.assertEqual(calls, [(u"hello, big world", 4321)])
+        self.assertEqual(calls, [("hello, big world", 4321)])
 
-    def test_validatingParameters(self):
-        # type: () -> None
+    def test_validatingParameters(self) -> None:
         """
         When a parameter fails to validate - for example, a non-number passed
         to a numeric Field, the request fails with a 400 and the default
@@ -436,8 +405,7 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(len(errors), 1)
         self.assertEquals(errors[0].text, "not a valid number")
 
-    def test_customParameterValidation(self):
-        # type: () -> None
+    def test_customParameterValidation(self) -> None:
         """
         When a custom parameter fails to validate by raising ValueError - for
         example, a non-number passed to a numeric Field, the request fails with
@@ -465,15 +433,12 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(len(errors), 1)
         errorText = cast(str, errors[0].text)
         self.assertIsNot(errorText, None)
-        self.assertTrue(
-            errorText.startswith("invalid literal for int() with base 10: ")
+        self.assertEqual(
+            errorText,
+            "invalid literal for int() with base 10: 'not a number'",
         )
-        # Peculiar 2-step assert because pypy2 (invalidly) sticks a 'u' in
-        # there.
-        self.assertTrue(errorText.endswith("'not a number'"))
 
-    def test_handlingJSON(self):
-        # type: () -> None
+    def test_handlingJSON(self) -> None:
         """
         A handler for a form with Fields receives those fields as input, as
         passed by an HTTP client that submits a JSON POST body.
@@ -490,15 +455,14 @@ class TestForms(SynchronousTestCase):
             stub.post(
                 "https://localhost/handle",
                 json=dict(name="hello", value="1234", ignoreme="extraneous"),
-                headers={u"X-Test-Session": session.identifier},
+                headers={"X-Test-Session": session.identifier},
             )
         )
         self.assertEqual(response.code, 200)
         self.assertEqual(self.successResultOf(content(response)), b"yay")
-        self.assertEqual(to.calls, [(u"hello", 1234)])
+        self.assertEqual(to.calls, [("hello", 1234)])
 
-    def test_missingOptionalParameterJSON(self):
-        # type: () -> None
+    def test_missingOptionalParameterJSON(self) -> None:
         """
         If a required Field is missing from the JSON body, its default value is
         used.
@@ -529,10 +493,9 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(response2.code, 200)
         self.assertEqual(self.successResultOf(content(response)), b"okay")
         self.assertEqual(self.successResultOf(content(response2)), b"okay")
-        self.assertEqual(to.calls, [(u"one", 7.0), (u"two", 2.0)])
+        self.assertEqual(to.calls, [("one", 7.0), ("two", 2.0)])
 
-    def test_rendering(self):
-        # type: () -> None
+    def test_rendering(self) -> None:
         """
         When a route requires form fields, it renders a form with those fields.
         """
@@ -562,8 +525,7 @@ class TestForms(SynchronousTestCase):
             submitButton[0].attrib["name"], "__klein_auto_submit__"
         )
 
-    def test_renderingExplicitSubmit(self):
-        # type: () -> None
+    def test_renderingExplicitSubmit(self) -> None:
         """
         When a form renderer specifies a submit button, no automatic submit
         button is rendered.
@@ -592,8 +554,7 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(len(submitButton), 1)
         self.assertEqual(submitButton[0].attrib["name"], "button")
 
-    def test_renderingFormGlue(self):
-        # type: () -> None
+    def test_renderingFormGlue(self) -> None:
         """
         When a form renderer renders just the glue, none of the rest of the
         form is included.
@@ -625,8 +586,7 @@ class TestForms(SynchronousTestCase):
         )
         self.assertEqual(protectionField[0].attrib["value"], session.identifier)
 
-    def test_renderingEmptyForm(self):
-        # type: () -> None
+    def test_renderingEmptyForm(self) -> None:
         """
         When a form renderer specifies a submit button, no automatic submit
         button is rendered.
@@ -661,10 +621,9 @@ class TestForms(SynchronousTestCase):
         )
         self.assertEqual(protectionField[0].attrib["value"], session.identifier)
 
-    def test_renderLookupError(self):
-        # type: () -> None
+    def test_renderLookupError(self) -> None:
         """
-        RenderableForm raises L{MissingRenderMethod} if anything attempst to
+        RenderableForm raises L{MissingRenderMethod} if anything attempts to
         look up a render method on it.
         """
         mem = MemorySessionStore()
@@ -686,8 +645,7 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(len(failures), 1)
         self.assertIn("MissingRenderMethod", str(failures[0]))
 
-    def test_customValidationHandling(self):
-        # type: () -> None
+    def test_customValidationHandling(self) -> None:
         """
         L{Form.onValidationFailureFor} handles form validation failures by
         handing its thing a renderable form.
@@ -721,8 +679,7 @@ class TestForms(SynchronousTestCase):
             [("value", 300)],
         )
 
-    def test_renderingWithNoSessionYet(self):
-        # type: () -> None
+    def test_renderingWithNoSessionYet(self) -> None:
         """
         When a route is rendered with no session, it sets a cookie to establish
         a new session.
@@ -732,14 +689,11 @@ class TestForms(SynchronousTestCase):
         response = self.successResultOf(stub.get("https://localhost/render"))
         self.assertEqual(response.code, 200)
         setCookie = response.cookies()["Klein-Secure-Session"]
-        expected = 'value="{}"'.format(setCookie)
-        actual = self.successResultOf(content(response))
-        if not isinstance(expected, bytes):  # type: ignore[unreachable]
-            actual = actual.decode("utf-8")
+        expected = f'value="{setCookie}"'
+        actual = self.successResultOf(content(response)).decode("utf-8")
         self.assertIn(expected, actual)
 
-    def test_noSessionPOST(self):
-        # type: () -> None
+    def test_noSessionPOST(self) -> None:
         """
         An unauthenticated, CSRF-protected form will return a 403 Forbidden
         status code.
@@ -757,8 +711,7 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(response.code, 403)
         self.assertIn(b"CSRF", self.successResultOf(content(response)))
 
-    def test_cookieNoToken(self):
-        # type: () -> None
+    def test_cookieNoToken(self) -> None:
         """
         A cookie-authenticated, CSRF-protected form will return a 403 Forbidden
         status code when a CSRF protection token is not supplied.
@@ -782,8 +735,7 @@ class TestForms(SynchronousTestCase):
         self.assertEqual(response.code, 403)
         self.assertIn(b"CSRF", self.successResultOf(content(response)))
 
-    def test_cookieWithToken(self):
-        # type: () -> None
+    def test_cookieWithToken(self) -> None:
         """
         A cookie-authenticated, CRSF-protected form will call the form as
         expected.
