@@ -2,31 +2,24 @@
 Tests for L{klein.plating}.
 """
 
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-    unicode_literals,
-)
 
 import json
 from string import printable
+from typing import Any
 
 import attr
-
-from hypothesis import given, settings, strategies as st
+from hypothesis import given, settings
+from hypothesis import strategies as st
 
 from twisted.internet.defer import Deferred, succeed
-from twisted.trial.unittest import (
-    SynchronousTestCase,
-    TestCase as AsynchronousTestCase,
-)
+from twisted.trial.unittest import SynchronousTestCase
+from twisted.trial.unittest import TestCase as AsynchronousTestCase
 from twisted.web.error import FlattenerError, MissingRenderMethod
 from twisted.web.template import slot, tags
 
-from .test_resource import _render, requestMock
 from .. import Klein, Plating
 from .._plating import ATOM_TYPES, PlatedElement, resolveDeferredObjects
+from .test_resource import MockRequest, _render
 
 
 page = Plating(
@@ -45,8 +38,14 @@ page = Plating(
 )
 
 element = Plating(
-    defaults={"a": "NO VALUE FOR A", "b": "NO VALUE FOR B",},
-    tags=tags.div(tags.span("a: ", slot("a")), tags.span("b: ", slot("b")),),
+    defaults={
+        "a": "NO VALUE FOR A",
+        "b": "NO VALUE FOR B",
+    },
+    tags=tags.div(
+        tags.span("a: ", slot("a")),
+        tags.span("b: ", slot("b")),
+    ),
 )
 
 
@@ -78,7 +77,7 @@ def registeredRenderMethod(*args):
     return tag("(self)" if len(args) == 3 else "", "some text!")
 
 
-class InstanceWidget(object):
+class InstanceWidget:
     """
     A class with a method that's a L{Plating.widget}.
     """
@@ -98,8 +97,8 @@ class InstanceWidget(object):
         return {"a": succeed(a), "b": succeed(b)}
 
 
-@attr.s
-class DeferredValue(object):
+@attr.s(auto_attribs=True)
+class DeferredValue:
     """
     A value within a JSON serializable object that is deferred.
 
@@ -108,8 +107,8 @@ class DeferredValue(object):
     @param deferred: The L{Deferred} representing the value.
     """
 
-    value = attr.ib()  # type: object
-    deferred = attr.ib(attr.Factory(Deferred))  # type: Deferred
+    value: Any
+    deferred: Deferred = attr.ib(factory=Deferred)
 
     def resolve(self):
         """
@@ -175,7 +174,7 @@ def transformJSONObject(jsonObject, transformer):
         elif isinstance(obj, dict):
             return {transformer(k): transformer(v) for k, v in obj.items()}
         else:
-            raise AssertionError("Object of unknown type {!r}".format(obj))
+            raise AssertionError(f"Object of unknown type {obj!r}")
 
     return visit(jsonObject)
 
@@ -243,7 +242,8 @@ class ResolveDeferredObjectsTests(SynchronousTestCase):
 
     @settings(max_examples=500)
     @given(
-        jsonObject=jsonObjects, data=st.data(),
+        jsonObject=jsonObjects,
+        data=st.data(),
     )
     def test_resolveObjects(self, jsonObject, data):
         """
@@ -262,10 +262,11 @@ class ResolveDeferredObjectsTests(SynchronousTestCase):
                 return value
 
         deferredJSONObject = transformJSONObject(
-            jsonObject, maybeWrapInDeferred,
+            jsonObject,
+            maybeWrapInDeferred,
         )
 
-        resolved = resolveDeferredObjects(deferredJSONObject)
+        resolved: "Deferred[Any]" = resolveDeferredObjects(deferredJSONObject)
 
         for value in deferredValues:
             value.resolve()
@@ -273,7 +274,8 @@ class ResolveDeferredObjectsTests(SynchronousTestCase):
         self.assertEqual(self.successResultOf(resolved), jsonObject)
 
     @given(
-        jsonObject=jsonObjects, data=st.data(),
+        jsonObject=jsonObjects,
+        data=st.data(),
     )
     def test_elementSerialized(self, jsonObject, data):
         """
@@ -295,10 +297,11 @@ class ResolveDeferredObjectsTests(SynchronousTestCase):
                 return value
 
         withPlatingElements = transformJSONObject(
-            jsonObject, injectPlatingElements,
+            jsonObject,
+            injectPlatingElements,
         )
 
-        resolved = resolveDeferredObjects(withPlatingElements)
+        resolved: "Deferred[Any]" = resolveDeferredObjects(withPlatingElements)
 
         self.assertEqual(self.successResultOf(resolved), jsonObject)
 
@@ -310,7 +313,7 @@ class ResolveDeferredObjectsTests(SynchronousTestCase):
         """
 
         @attr.s
-        class ConsistentRepr(object):
+        class ConsistentRepr:
             """
             Objects with a predictable repr
             """
@@ -340,7 +343,7 @@ class PlatingTests(AsynchronousTestCase):
         succeed synchronously, and return the generated request object and
         written bytes.
         """
-        request = requestMock(uri)
+        request = MockRequest(uri)
         d = _render(self.kr, request)
         self.successResultOf(d)
         return request, request.getWrittenData()
@@ -366,7 +369,7 @@ class PlatingTests(AsynchronousTestCase):
         in the decorated method receiving the appropriate C{self}.
         """
 
-        class AppObj(object):
+        class AppObj:
             app = Klein()
 
             def __init__(self, x):
@@ -397,7 +400,7 @@ class PlatingTests(AsynchronousTestCase):
         request, written = self.get(b"/?json=true")
         self.assertEqual(
             request.responseHeaders.getRawHeaders(b"content-type")[0],
-            b"text/json; charset=utf-8",
+            b"application/json",
         )
         self.assertEquals(
             {"ok": "an-plating-test", "title": "default title unchanged"},
@@ -418,7 +421,7 @@ class PlatingTests(AsynchronousTestCase):
         request, written = self.get(b"/?json=true")
         self.assertEqual(
             request.responseHeaders.getRawHeaders(b"content-type")[0],
-            b"text/json; charset=utf-8",
+            b"application/json",
         )
         self.assertEquals(
             {"ok": "an-plating-test", "title": "default title unchanged"},
