@@ -2,29 +2,23 @@
 Tests for L{klein._session}.
 """
 
-from typing import TYPE_CHECKING
+from typing import Any, Generator, List, Tuple, Type
 
 from treq.testing import StubTreq
-
-from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.trial.unittest import SynchronousTestCase
-
 from zope.interface import Interface, implementer
 
+from twisted.internet.defer import Deferred, inlineCallbacks
+from twisted.python.components import Componentized
+from twisted.trial.unittest import SynchronousTestCase
+from twisted.web.iweb import IRequest
+
 from klein import Authorization, Klein, Requirer, SessionProcurer
-from klein._typing import ifmethod
 from klein.interfaces import ISession, NoSuchSession, TooLateForCookies
 from klein.storage.memory import MemorySessionStore, declareMemoryAuthorizer
 
-if TYPE_CHECKING:  # pragma: no cover
-    from twisted.web.iweb import IRequest
-    from twisted.internet.defer import Deferred
-    from twisted.python.components import Componentized
-    from zope.interface.interfaces import IInterface
-    from typing import Tuple, List
 
-    sessions = List[ISession]
-    errors = List[NoSuchSession]
+Sessions = List[ISession]
+Errors = List[NoSuchSession]
 
 
 class ISimpleTest(Interface):
@@ -32,9 +26,7 @@ class ISimpleTest(Interface):
     Interface for testing.
     """
 
-    @ifmethod
-    def doTest():
-        # type: () -> None
+    def doTest() -> int:
         """
         Test method.
         """
@@ -47,13 +39,12 @@ class IDenyMe(Interface):
 
 
 @implementer(ISimpleTest)
-class SimpleTest(object):
+class SimpleTest:
     """
     Implementation of L{ISimpleTest} for testing.
     """
 
-    def doTest(self):
-        # type: () -> int
+    def doTest(self) -> int:
         """
         Implementation of L{ISimpleTest}.  Returns 3.
         """
@@ -61,21 +52,21 @@ class SimpleTest(object):
 
 
 @declareMemoryAuthorizer(ISimpleTest)
-def memoryAuthorizer(interface, session, data):
-    # type: (IInterface, ISession, Componentized) -> SimpleTest
+def memoryAuthorizer(
+    interface: Type[Interface], session: ISession, data: Componentized
+) -> SimpleTest:
     """
     Authorize the ISimpleTest interface; it always works.
     """
     return SimpleTest()
 
 
-def simpleSessionRouter():
-    # type: () -> Tuple[sessions, errors, str, str, StubTreq]
+def simpleSessionRouter() -> Tuple[Sessions, Errors, str, str, StubTreq]:
     """
     Construct a simple router.
     """
-    sessions = []
-    exceptions = []
+    sessions: Sessions = []
+    exceptions: Errors = []
     mss = MemorySessionStore.fromAuthorizers([memoryAuthorizer])
     router = Klein()
     token = "X-Test-Session-Token"
@@ -88,29 +79,27 @@ def simpleSessionRouter():
 
     @router.route("/")
     @inlineCallbacks
-    def route(request):
-        # type: (IRequest) -> Deferred
+    def route(request: IRequest) -> Generator[Any, object, bytes]:
         try:
-            sessions.append((yield sproc.procureSession(request)))
+            sessions.append(
+                (yield sproc.procureSession(request)),  # type: ignore[arg-type]
+            )
         except NoSuchSession as nss:
             exceptions.append(nss)
-        returnValue(b"ok")
+        return b"ok"
 
     requirer = Requirer()
 
     @requirer.prerequisite([ISession])
-    def procure(request):
-        # type: (IRequest) -> Deferred
+    def procure(request: IRequest) -> Deferred:
         return sproc.procureSession(request)
 
     @requirer.require(router.route("/test"), simple=Authorization(ISimpleTest))
-    def testRoute(simple):
-        # type: (SimpleTest) -> str
+    def testRoute(simple: SimpleTest) -> str:
         return "ok: " + str(simple.doTest() + 4)
 
     @requirer.require(router.route("/denied"), nope=Authorization(IDenyMe))
-    def testDenied(nope):
-        # type: (IDenyMe) -> str
+    def testDenied(nope: IDenyMe) -> str:
         return "bad"
 
     treq = StubTreq(router.resource())
@@ -122,8 +111,7 @@ class ProcurementTests(SynchronousTestCase):
     Tests for L{klein.SessionProcurer}.
     """
 
-    def test_procurementSecurity(self):
-        # type: () -> None
+    def test_procurementSecurity(self) -> None:
         """
         Once a session is negotiated, it should be the identical object to
         avoid duplicate work - unless we are using forceInsecure to retrieve
@@ -136,15 +124,14 @@ class ProcurementTests(SynchronousTestCase):
 
         @router.route("/")
         @inlineCallbacks
-        def route(request):
-            # type: (IRequest) -> Deferred
+        def route(request: IRequest) -> Generator[Any, object, bytes]:
             sproc = SessionProcurer(mss)
             sessions.append((yield sproc.procureSession(request)))
             sessions.append((yield sproc.procureSession(request)))
             sessions.append(
                 (yield sproc.procureSession(request, forceInsecure=True))
             )
-            returnValue(b"sessioned")
+            return b"sessioned"
 
         treq = StubTreq(router.resource())
         self.successResultOf(treq.get("http://unittest.example.com/"))
@@ -154,8 +141,7 @@ class ProcurementTests(SynchronousTestCase):
         self.assertIs(sessions[3], sessions[4])
         self.assertIsNot(sessions[3], sessions[5])
 
-    def test_procuredTooLate(self):
-        # type: () -> None
+    def test_procuredTooLate(self) -> None:
         """
         If you start writing stuff to the response before procuring the
         session, when cookies need to be set, you will get a comprehensible
@@ -164,10 +150,9 @@ class ProcurementTests(SynchronousTestCase):
         mss = MemorySessionStore()
         router = Klein()
 
-        @router.route("/")
+        @router.route("/")  # type: ignore[arg-type]
         @inlineCallbacks
-        def route(request):
-            # type: (IRequest) -> Deferred
+        def route(request: IRequest) -> Generator[Any, object, None]:
             sproc = SessionProcurer(mss)
             request.write(b"oops...")
             with self.assertRaises(TooLateForCookies):
@@ -179,8 +164,7 @@ class ProcurementTests(SynchronousTestCase):
         result = self.successResultOf(treq.get("http://unittest.example.com/"))
         self.assertEqual(self.successResultOf(result.content()), b"oops...bye")
 
-    def test_cookiesTurnedOff(self):
-        # type: () -> None
+    def test_cookiesTurnedOff(self) -> None:
         """
         If cookies can't be set, then C{procureSession} raises
         L{NoSuchSession}.
@@ -190,19 +174,17 @@ class ProcurementTests(SynchronousTestCase):
 
         @router.route("/")
         @inlineCallbacks
-        def route(request):
-            # type: (IRequest) -> Deferred
+        def route(request: IRequest) -> Generator[Any, object, bytes]:
             sproc = SessionProcurer(mss, setCookieOnGET=False)
             with self.assertRaises(NoSuchSession):
                 yield sproc.procureSession(request)
-            returnValue(b"no session")
+            return b"no session"
 
         treq = StubTreq(router.resource())
         result = self.successResultOf(treq.get("http://unittest.example.com/"))
         self.assertEqual(self.successResultOf(result.content()), b"no session")
 
-    def test_unknownSessionHeader(self):
-        # type: () -> None
+    def test_unknownSessionHeader(self) -> None:
         """
         Unknown session IDs in auth headers will be immediately rejected with
         L{NoSuchSession}.
@@ -210,14 +192,13 @@ class ProcurementTests(SynchronousTestCase):
         sessions, exceptions, token, cookie, treq = simpleSessionRouter()
 
         response = self.successResultOf(
-            treq.get("https://unittest.example.com/", headers={token: u"bad"})
+            treq.get("https://unittest.example.com/", headers={token: "bad"})
         )
         self.assertEqual(response.code, 200)
         self.assertEqual(len(sessions), 0)
         self.assertEqual(len(exceptions), 1)
 
-    def test_unknownSessionCookieGET(self):
-        # type: () -> None
+    def test_unknownSessionCookieGET(self) -> None:
         """
         Unknown session IDs in cookies will result in a new session being
         created.
@@ -234,8 +215,7 @@ class ProcurementTests(SynchronousTestCase):
         self.assertEqual(len(sessions), 1)
         self.assertNotEqual(sessions[0].identifier, badSessionID)
 
-    def test_unknownSessionCookiePOST(self):
-        # type: () -> None
+    def test_unknownSessionCookiePOST(self) -> None:
         """
         Unknown session IDs in cookies for POST requests will result in a
         NoSuchSession error.
@@ -251,8 +231,7 @@ class ProcurementTests(SynchronousTestCase):
         self.assertEqual(len(exceptions), 1)
         self.assertEqual(len(sessions), 0)
 
-    def test_authorization(self):
-        # type: () -> None
+    def test_authorization(self) -> None:
         """
         When L{Requirer.require} is used with L{Authorization} and the session
         knows how to supply that authorization, it is passed to the object.
@@ -263,8 +242,7 @@ class ProcurementTests(SynchronousTestCase):
         )
         self.assertEqual(self.successResultOf(response.content()), b"ok: 7")
 
-    def test_authorizationDenied(self):
-        # type: () -> None
+    def test_authorizationDenied(self) -> None:
         """
         When L{Requirer.require} is used with an L{Authorization} and the
         session does I{not} know how to supply that authorization, the callable
