@@ -2,132 +2,138 @@
 Shared tools for Klein's test suite.
 """
 
+from abc import ABC, abstractmethod
+from typing import Type, TypeVar, cast
+
+from twisted.internet.defer import Deferred
+from twisted.trial.unittest import SynchronousTestCase
 
 
-class EqualityTestsMixin(object):
+class EqualityTestsMixin(ABC):
     """
     A mixin defining tests for the standard implementation of C{==} and C{!=}.
     """
-    def anInstance(self):
+
+    @abstractmethod
+    def anInstance(self) -> object:
         """
         Return an instance of the class under test.  Each call to this method
         must return a different object.  All objects returned must be equal to
         each other.
         """
-        raise NotImplementedError()
 
-
-    def anotherInstance(self):
+    @abstractmethod
+    def anotherInstance(self) -> object:
         """
         Return an instance of the class under test.  Each call to this method
         must return a different object.  The objects must not be equal to the
         objects returned by C{anInstance}.  They may or may not be equal to
         each other (they will not be compared against each other).
         """
-        raise NotImplementedError()
 
-
-    def test_identicalEq(self):
+    def test_identicalEq(self) -> None:
         """
         An object compares equal to itself using the C{==} operator.
         """
         o = self.anInstance()
-        self.assertTrue(o == o)
+        cast(SynchronousTestCase, self).assertTrue(o == o)
 
-
-    def test_identicalNe(self):
+    def test_identicalNe(self) -> None:
         """
         An object doesn't compare not equal to itself using the C{!=} operator.
         """
         o = self.anInstance()
-        self.assertFalse(o != o)
+        cast(SynchronousTestCase, self).assertFalse(o != o)
 
-
-    def test_sameEq(self):
+    def test_sameEq(self) -> None:
         """
         Two objects that are equal to each other compare equal to each other
         using the C{==} operator.
         """
         a = self.anInstance()
         b = self.anInstance()
-        self.assertTrue(a == b)
+        cast(SynchronousTestCase, self).assertTrue(a == b)
 
-
-    def test_sameNe(self):
+    def test_sameNe(self) -> None:
         """
         Two objects that are equal to each other do not compare not equal to
         each other using the C{!=} operator.
         """
         a = self.anInstance()
         b = self.anInstance()
-        self.assertFalse(a != b)
+        cast(SynchronousTestCase, self).assertFalse(a != b)
 
-
-    def test_differentEq(self):
+    def test_differentEq(self) -> None:
         """
         Two objects that are not equal to each other do not compare equal to
         each other using the C{==} operator.
         """
         a = self.anInstance()
         b = self.anotherInstance()
-        self.assertFalse(a == b)
+        cast(SynchronousTestCase, self).assertFalse(a == b)
 
-
-    def test_differentNe(self):
+    def test_differentNe(self) -> None:
         """
         Two objects that are not equal to each other compare not equal to each
         other using the C{!=} operator.
         """
         a = self.anInstance()
         b = self.anotherInstance()
-        self.assertTrue(a != b)
+        cast(SynchronousTestCase, self).assertTrue(a != b)
 
-
-    def test_anotherTypeEq(self):
+    def test_anotherTypeEq(self) -> None:
         """
         The object does not compare equal to an object of an unrelated type
         (which does not implement the comparison) using the C{==} operator.
         """
         a = self.anInstance()
         b = object()
-        self.assertFalse(a == b)
+        cast(SynchronousTestCase, self).assertFalse(a == b)
 
-
-    def test_anotherTypeNe(self):
+    def test_anotherTypeNe(self) -> None:
         """
         The object compares not equal to an object of an unrelated type (which
         does not implement the comparison) using the C{!=} operator.
         """
         a = self.anInstance()
         b = object()
-        self.assertTrue(a != b)
+        cast(SynchronousTestCase, self).assertTrue(a != b)
 
-
-    def test_delegatedEq(self):
+    def test_delegatedEq(self) -> None:
         """
         The result of comparison using C{==} is delegated to the right-hand
         operand if it is of an unrelated type.
         """
-        class Delegate(object):
-            def __eq__(self, other):
-                # Do something crazy and obvious.
-                return [self]
+
+        class Delegate:
+            def __eq__(self, other: object) -> bool:
+                # Do something crazy and easily identifiable.
+                return cast(bool, [self])
 
         a = self.anInstance()
         b = Delegate()
-        self.assertEqual(a == b, [b])
+        cast(SynchronousTestCase, self).assertEqual(a == b, [b])
 
-
-    def test_delegateNe(self):
+    def test_delegateNe(self) -> None:
         """
         The result of comparison using C{!=} is delegated to the right-hand
         operand if it is of an unrelated type.
         """
-        class Delegate(object):
-            def __ne__(self, other):
-                # Do something crazy and obvious.
-                return [self]
+
+        class Delegate:
+            def __ne__(self, other: object) -> bool:
+                # Do something crazy and easily identifiable.
+                return cast(bool, [self])
 
         a = self.anInstance()
         b = Delegate()
-        self.assertEqual(a != b, [b])
+        cast(SynchronousTestCase, self).assertEqual(a != b, [b])
+
+
+_T = TypeVar("_T")
+
+
+def recover(d: "Deferred[_T]", exc_type: Type[Exception]) -> "Deferred[_T]":
+    return d.addErrback(
+        lambda f: f.trap(exc_type)  # type: ignore[no-any-return]
+    )
